<link rel="stylesheet" href="{{ STATIC_URL }}storage/reqs/leaflet/leaflet.css" />
<link rel="stylesheet" href="{{ STATIC_URL }}storage/reqs/markercluster/MarkerCluster.css" />
<link rel="stylesheet" href="{{ STATIC_URL }}storage/reqs/markercluster/MarkerCluster.Default.css" />
<link rel="stylesheet" href="{{ STATIC_URL }}storage/reqs/draw/leaflet.draw.css" />
<link rel="stylesheet" href="{{ STATIC_URL }}storage/reqs/editinosm/Leaflet.EditInOSM.css" />
<link rel="stylesheet" href="{{ STATIC_URL }}storage/reqs/minimap/Control.MiniMap.css" />
<link rel="stylesheet" href="{{ STATIC_URL }}storage/reqs/contextmenu/leaflet.contextmenu.css" />
<link rel="stylesheet" href="{{ STATIC_URL }}storage/reqs/label/leaflet.label.css" />
<link rel="stylesheet" href="{{ STATIC_URL }}storage/src/css/storage.css" />
