{% load i18n %}
<h5>{% trans "Please log in to proceed" %}</h5>
{% if form.errors %}
    <p>{% trans "Your username and password didn't match. Please try again." %}</p>
{% endif %}

<form method="post" action="{% url 'login' %}" id="login_form">
    {% csrf_token %}
    <table>
    <tr>
        <td>{{ form.username.label_tag }}</td>
        <td>{{ form.username }}</td>
    </tr>
    <tr>
        <td>{{ form.password.label_tag }}</td>
        <td>{{ form.password }}</td>
    </tr>
    </table>

    <input type="submit" value="login" />
    <input type="hidden" name="next" value="{{ next }}" />
</form>