"Create collaborative maps on top of Geodjango and Leaflet."
VERSION = (0, 6, 8)

__author__ = 'Yohan Boniface'
__contact__ = "yb@enix.fr"
__homepage__ = "https://github.com/yohanboniface/django-leaflet-storage"
__version__ = ".".join(map(str, VERSION))
